package com.capitalware.ufmsm.mq;

import java.io.IOException;
import java.net.MalformedURLException;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;

import javax.swing.*;

import com.capitalware.ufmsm.UFMSM;
import com.capitalware.ufmsm.mq.objects.MQObject;

import com.ibm.mq.*;

/**
 * This class will handle getting one message message from the status queue.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MQGetOneMessage extends MyMQ
{
   private UFMSM            parent;
   private MQQueue               inQ = null;

   /**
    * The constructor
    *
    * @param cpp contains Communication Profile Parameters object
    * @param parent The parent object
    */
   public MQGetOneMessage(MQObject mqo, UFMSM parent)
   {
      super(mqo);
      this.parent = parent;
   }

   /**
    * Connect to the queue manager and open the status queue.
    */
   public boolean connectAndOpenQ()
   {
      boolean majorFailure = false;
      String  popupTitle = "";
      String  popupMessage = "";
      int openInputOptions  = MQC.MQOO_INQUIRE + MQC.MQOO_BROWSE + MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_INPUT_SHARED;

      try
      {
         connect();
         
         inQ = _qMgr.accessQueue( mqo.qObject.qName,
                                  openInputOptions,
                                  null,
                                  null,
                                  null );
         
      }
      catch (MQException e)
      {
         popupTitle = "Error";
         popupMessage = "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode;
         majorFailure = true;
      }
      catch (MalformedURLException e)
      {
         popupTitle = "Error: MalformedURLException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (InvalidAlgorithmParameterException e)
      {
         popupTitle = "Error: InvalidAlgorithmParameterException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (NoSuchAlgorithmException e)
      {
         popupTitle = "Error: NoSuchAlgorithmException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (ClassNotFoundException e)
      {
         popupTitle = "Error: ClassNotFoundException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (InstantiationException e)
      {
         popupTitle = "Error: InstantiationException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (IllegalAccessException e)
      {
         popupTitle = "Error: IllegalAccessException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      finally
      {
         /**
          * Hard exit now that the user knows what has happened.
          */
         if (majorFailure)
         {
            JOptionPane.showMessageDialog( parent,
                                           popupMessage,
                                           popupTitle,
                                           JOptionPane.ERROR_MESSAGE);
            System.exit(99);
         }
      }

      return majorFailure;
   }

   /**
    * Close the status queue and disconnect from the queue manager.
    */
   public void closeQAndDisconnect()
   {
      try
      {
         if (inQ != null)
            inQ.close();
      }
      catch (MQException e)
      {
         JOptionPane.showMessageDialog( parent,
                                        "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode,
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
      }
      
      try
      {
         if (_qMgr != null)
            _qMgr.disconnect();
      }
      catch (MQException e)
      {
         JOptionPane.showMessageDialog( parent,
                                        "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode,
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
      }
   }

   /**
    * Get the message.
    */
   public byte[] getMessage(byte[] correlID)
   {
      byte[] byteData = null;

      MQGetMessageOptions getOptions = new MQGetMessageOptions();
      getOptions.options = MQC.MQGMO_BROWSE_FIRST + MQC.MQGMO_CONVERT + MQC.MQGMO_NO_WAIT + MQC.MQGMO_FAIL_IF_QUIESCING;
      getOptions.matchOptions = MQC.MQMO_MATCH_CORREL_ID;

      MQMessage getMsg = new MQMessage();
      getMsg.correlationId = correlID;

      try
      {
         inQ.get(getMsg, getOptions);

         byteData = new byte[getMsg.getMessageLength()];
         getMsg.readFully(byteData);
      }
      catch (IOException e)
      {
         JOptionPane.showMessageDialog( parent,
                                       "IOException: " + e.getMessage(),
                                       "Error",
                                       JOptionPane.ERROR_MESSAGE);
      }
      catch (MQException e)
      {
         if (e.completionCode == 2 && e.reasonCode == MQException.MQRC_NO_MSG_AVAILABLE)
         {
            JOptionPane.showMessageDialog( parent,
                                           "Message not found in status queue.",
                                           "Error",
                                           JOptionPane.ERROR_MESSAGE);
         }
         else
         {
            JOptionPane.showMessageDialog( parent,
                                           "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode,
                                           "Error",
                                           JOptionPane.ERROR_MESSAGE);
         }
      }

      return byteData;
   }
}
